@@ -2,7 +2,7 @@
  * SSLsplit - transparent SSL/TLS interception
  * https://www.roe.ch/SSLsplit
  *
- * Copyright (c) 2009-2018, Daniel Roethlisberger <daniel@roe.ch>.
+ * Copyright (c) 2009-2019, Daniel Roethlisberger <daniel@roe.ch>.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -63,7 +63,7 @@ void
 log_exceptcb(void)
 {
 	if (proxy_ctx) {
-		proxy_loopbreak(proxy_ctx);
+		proxy_loopbreak(proxy_ctx, -1);
 		proxy_ctx = NULL;
 	}
 }
