@@ -2,7 +2,7 @@
  * SSLsplit - transparent SSL/TLS interception
  * https://www.roe.ch/SSLsplit
  *
- * Copyright (c) 2009-2018, Daniel Roethlisberger <daniel@roe.ch>.
+ * Copyright (c) 2009-2019, Daniel Roethlisberger <daniel@roe.ch>.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -774,10 +774,12 @@ sys_sendmsgfd(int sock, void *buf, size_t bufsz, int fd)
 	msg.msg_namelen = 0;
 	msg.msg_iov = &iov;
 	msg.msg_iovlen = 1;
+	msg.msg_flags = 0;
 
 	if (fd != -1) {
 		msg.msg_control = cmsgbuf;
 		msg.msg_controllen = sizeof(cmsgbuf);
+		memset(cmsgbuf, 0, sizeof(cmsgbuf));
 
 		cmsg = CMSG_FIRSTHDR(&msg);
 		if (!cmsg)
